using System;
using System.Web.UI;

using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.SiteProvider;
using CMS.TreeEngine;
using CMS.UIControls;
using CMS.WorkflowEngine;
using CMS.FormControls;

public partial class CMSDesk_Content_Properties_metadata : CMSDeskPage
{
    #region "Protected variables"

    protected int nodeId = 0;
    protected TreeNode node = null;
    protected TreeProvider tree = null;
    protected bool hasModifyPermission = true;
    FormEngineUserControl tagSelectorElem = null;
    FormEngineUserControl tagGroupSelectorElem = null;

    #endregion


    #region "Properties"

    /// <summary>
    /// Indicates whether the asynchronous postback occurs on the page.
    /// </summary>
    private bool IsAsyncPostback
    {
        get
        {
            return ScriptManager.GetCurrent(this.Page).IsInAsyncPostBack;
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        // Register the scripts
        ScriptHelper.RegisterProgress(this.Page);
        
        tagGroupSelectorElem = this.Page.LoadControl("~/CMSFormControls/TagGroupSelector.ascx") as FormEngineUserControl;
        tagGroupSelectorElem.ID = "ctrlTagGroupSelector";
        tagGroupSelectorElem.SetValue("AddNoneItemsRecord", false);
        tagGroupSelectorElem.SetValue("UseAutoPostback", true);
        tagGroupSelectorElem.SetValue("UseGroupNameForSelection", false);
        plcTagGroupSelector.Controls.Add(tagGroupSelectorElem);

        tagSelectorElem = this.Page.LoadControl("~/CMSFormControls/TagSelector.ascx") as FormEngineUserControl;
        tagSelectorElem.ID = "ctrlTagSelector";
        tagSelectorElem.IsLiveSite = false;
        plcTagSelector.Controls.Add(tagSelectorElem);

        UIContext.PropertyTab = PropertyTabEnum.Metadata;

        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);

        // Get the document
        tree = new TreeProvider(CMSContext.CurrentUser);
        node = DocumentHelper.GetDocument(nodeId, CMSContext.PreferredCultureCode, tree);

        SetCheckBoxes();

        ltlScript.Text = "";

        lblPageDescription.Text = ResHelper.GetString("PageProperties.Decription");
        lblPageKeywords.Text = ResHelper.GetString("PageProperties.Keywords");
        lblPageTitle.Text = ResHelper.GetString("PageProperties.Title");
        lblTagSelector.Text = ResHelper.GetString("PageProperties.Tags");
        lblTagGroupSelector.Text = ResHelper.GetString("PageProperties.TagGroup");
        chkDescription.Text = ResHelper.GetString("Metadata.Inherit");
        chkKeyWords.Text = ResHelper.GetString("Metadata.Inherit");
        chkTitle.Text = ResHelper.GetString("Metadata.Inherit");
        chkTagGroupSelector.Text = ResHelper.GetString("Metadata.Inherit");

        btnOK.Text = ResHelper.GetString("general.ok");

        menuElem.OnBeforeCheckIn += new EventHandler(menuElem_OnBeforeCheckIn);
        menuElem.OnBeforeApprove += new EventHandler(menuElem_OnBeforeApprove);
        menuElem.OnBeforeReject += new EventHandler(menuElem_OnBeforeReject);
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        if (menuElem.AllowEdit && menuElem.AllowSave && hasModifyPermission)
        {
            Page.ClientScript.RegisterClientScriptBlock(typeof(string), "EditShortcuts",
                "<script type=\"text/javascript\" src=\"" + ResolveUrl("~/CMSScripts/shortcuts.js") + "\"></script>"
            );
            pnlForm.Enabled = true;
        }
        else
        {
            pnlForm.Enabled = false;
        }

        lblWorkflowInfo.Text = menuElem.WorkflowInfo;
        lblWorkflowInfo.Visible = (lblWorkflowInfo.Text != "");

        if (DataHelper.DataSourceIsEmpty(TagGroupInfoProvider.GetTagGroups(CMSContext.CurrentSiteID)))
        {
            // Hide tag controls
            lblTagGroupSelector.Visible = false;
            tagGroupSelectorElem.Visible = false;
            chkTagGroupSelector.Visible = false;
            lblTagSelector.Visible = false;
            tagSelectorElem.Visible = false;

            // Show tag info
            lblTagInfo.Text = ResHelper.GetString("PageProperties.TagsInfo");
            lblTagInfo.Visible = true;
        }
        else
        {
            // Set tag group Id to tag selector
            int tagGroupId = ValidationHelper.GetInteger(tagGroupSelectorElem.Value, 0);
            if (tagGroupId > 0)
            {
                tagSelectorElem.Enabled = true;
                tagSelectorElem.SetValue("GroupID", tagGroupId);
            }
            else
            {
                tagSelectorElem.Enabled = false;
            }
        }

        // Get the document with valid data if under the workflow
        if (lblWorkflowInfo.Visible)
        {
            node = DocumentHelper.GetDocument(nodeId, CMSContext.PreferredCultureCode, tree);
        }

        // Display none record when there is no tag group given
        if ((node.DocumentTagGroupID == 0) && (ValidationHelper.GetInteger(node.GetInheritedValue("DocumentTagGroupID", false), 0) == 0))
        {
            tagGroupSelectorElem.SetValue("AddNoneItemsRecord", true);
            if (!UrlHelper.IsPostback())
            {
                tagSelectorElem.Enabled = false;
            }
        }

        if (!IsAsyncPostback)
        {
            ReloadData();
        }
    }

    #endregion


    #region "Protected methods"

    protected void SetCheckBoxes()
    {
        if (RequestHelper.IsPostBack())
        {
            return;
        }

        if ((node != null) && (node.NodeAliasPath == "/"))
        {
            chkDescription.Visible = false;
            chkKeyWords.Visible = false;
            chkTitle.Visible = false;
            chkTagGroupSelector.Visible = false;
        }
    }


    private void ReloadData()
    {
        if (node != null)
        {
            bool isRoot = (node.NodeAliasPath == "/");

            // Check read permissions
            if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
            }
            // Check modify permissions
            else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
            {
                hasModifyPermission = false;
                pnlForm.Enabled = false;
                lblWorkflowInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
            }

            if (node.GetValue("DocumentPageTitle") != null)
            {
                txtTitle.Text = node.GetValue("DocumentPageTitle").ToString();
            }
            else
            {
                if (!isRoot)
                {
                    txtTitle.Enabled = false;
                    chkTitle.Checked = true;
                    txtTitle.Text = ValidationHelper.GetString(node.GetInheritedValue("DocumentPageTitle", false), "");
                }
            }

            if (node.GetValue("DocumentPageKeyWords") != null)
            {
                txtKeywords.Text = node.GetValue("DocumentPageKeyWords").ToString();
            }
            else
            {
                if (!isRoot)
                {
                    txtKeywords.Enabled = false;
                    chkKeyWords.Checked = true;
                    txtKeywords.Text = ValidationHelper.GetString(node.GetInheritedValue("DocumentPageKeyWords", false), "");
                }
            }

            if (node.GetValue("DocumentPageDescription") != null)
            {
                txtDescription.Text = node.GetValue("DocumentPageDescription").ToString();
            }
            else
            {
                if (!isRoot)
                {
                    txtDescription.Enabled = false;
                    chkDescription.Checked = true;
                    txtDescription.Text = ValidationHelper.GetString(node.GetInheritedValue("DocumentPageDescription", false), "");
                }
            }

            if (node.GetValue("DocumentTagGroupID") != null)
            {
                object tagGroupId = node.GetValue("DocumentTagGroupID");
                tagGroupSelectorElem.Value = tagGroupId;
                tagSelectorElem.SetValue("GroupID", tagGroupId);
            }
            else
            {
                if (!isRoot)
                {
                    // Get the inherited tag group
                    int tagGroup = ValidationHelper.GetInteger(node.GetInheritedValue("DocumentTagGroupID", false), 0);
                    if (tagGroup > 0)
                    {
                        tagGroupSelectorElem.Value = tagGroup;
                        tagSelectorElem.SetValue("GroupID", tagGroup);
                    }
                    tagGroupSelectorElem.Enabled = false;
                    chkTagGroupSelector.Checked = true;
                }
            }

            tagSelectorElem.Value = node.DocumentTags;
        }
    }

    #endregion


    #region "Control handling"

    void menuElem_OnBeforeCheckIn(object sender, EventArgs e)
    {
        btnOK_Click(sender, e);
        lblInfo.Text += "<br />" + ResHelper.GetString("ContentEdit.WasCheckedIn");
    }


    void menuElem_OnBeforeReject(object sender, EventArgs e)
    {
        btnOK_Click(sender, e);
    }


    void menuElem_OnBeforeApprove(object sender, EventArgs e)
    {
        btnOK_Click(sender, e);
    }


    /// <summary>
    /// Updates node properties in database
    /// </summary>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        if (node != null)
        {

            bool autoCheck = !VersionManager.UseCheckInCheckOut(CMSContext.CurrentSiteName);
            VersionManager vm = null;
            bool useWorkflow = false;
            WorkflowManager workflowMan = new WorkflowManager(tree);

            // Get the document
            node = DocumentHelper.GetDocument(nodeId, CMSContext.PreferredCultureCode, tree);

            //test if the document uses workflow
            if (workflowMan.GetNodeWorkflowScope(node) != null)
            {
                useWorkflow = true;
            }

            // If not using check-in/check-out, check out automatically
            if (autoCheck & useWorkflow)
            {
                if (node != null)
                {
                    // Check out
                    vm = new VersionManager(tree);
                    vm.CheckOut(node);
                }
            }

            // Update the data
            if (node != null)
            {
                node.SetValue("DocumentPageTitle", null);
                if (!chkTitle.Checked)
                {
                    node.SetValue("DocumentPageTitle", txtTitle.Text);
                }


                node.SetValue("DocumentPageKeyWords", null);
                if (!chkKeyWords.Checked)
                {
                    node.SetValue("DocumentPageKeyWords", txtKeywords.Text);
                }

                node.SetValue("DocumentPageDescription", null);
                if (!chkDescription.Checked)
                {
                    node.SetValue("DocumentPageDescription", txtDescription.Text);
                }

                node.SetValue("DocumentTagGroupID", null);
                if (!chkTagGroupSelector.Checked)
                {
                    // If is tag group id
                    if (ValidationHelper.GetInteger(tagGroupSelectorElem.Value, 0) > 0)
                    {
                        node.SetValue("DocumentTagGroupID", tagGroupSelectorElem.Value);
                    }
                    else
                    {
                        node.SetValue("DocumentTagGroupID", DBNull.Value);
                    }
                }

                if (ValidationHelper.GetInteger(tagGroupSelectorElem.Value, 0) > 0)
                {
                    node.SetValue("DocumentTags", tagSelectorElem.Value);
                }
                else
                {
                    node.SetValue("DocumentTags", DBNull.Value);
                }

                DocumentHelper.UpdateDocument(node, tree);

                // Check in the document
                if (autoCheck & useWorkflow)
                {
                    if (vm != null)
                    {
                        vm.CheckIn(node, null, null);
                    }
                }
            }

            menuElem.ReloadMenu();

            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("General.ChangesSaved");

            // Reload tags
            if (node != null)
            {
                tagSelectorElem.Value = node.DocumentTags;
            }
        }
    }


    protected void chkTitle_CheckedChanged(object sender, EventArgs e)
    {
        if (chkTitle.Checked)
        {
            txtTitle.Enabled = false;
            txtTitle.Text = ValidationHelper.GetString(node.GetInheritedValue("DocumentPageTitle", false), "");
        }
        else
        {
            txtTitle.Enabled = true;
        }
    }


    protected void chkDescription_CheckedChanged(object sender, EventArgs e)
    {
        if (chkDescription.Checked)
        {
            txtDescription.Enabled = false;
            txtDescription.Text = ValidationHelper.GetString(node.GetInheritedValue("DocumentPageDescription", false), "");
        }
        else
        {
            txtDescription.Enabled = true;
        }
    }


    protected void chkKeyWords_CheckedChanged(object sender, EventArgs e)
    {
        if (chkKeyWords.Checked)
        {
            txtKeywords.Enabled = false;
            txtKeywords.Text = ValidationHelper.GetString(node.GetInheritedValue("DocumentPageKeyWords", false), "");
        }
        else
        {
            txtKeywords.Enabled = true;
        }
    }


    protected void chkTagGroupSelector_CheckedChanged(object sender, EventArgs e)
    {
        if (chkTagGroupSelector.Checked)
        {
            tagGroupSelectorElem.Enabled = false;

            int value = ValidationHelper.GetInteger(node.GetInheritedValue("DocumentTagGroupID", false), 0);
            if (value == 0)
            {
                tagGroupSelectorElem.SetValue("AddNoneItemsRecord", true);
            }
            tagGroupSelectorElem.Value = value;
        }
        else
        {
            tagGroupSelectorElem.Enabled = true;
        }
    }

    #endregion
}
