using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Reflection;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.LicenseProvider;
using CMS.UIControls;
using CMS.SettingsProvider;
using CMS.SiteProvider;

public partial class CMSModules_SmartSearch_Controls_UI_SearchIndex_New : CMSAdminEditControl
{
    protected void Page_Load(object sender, EventArgs e)
    {
        // Check read permission
        if (!CheckPermissions("cms.searchindex", CMSAdminControl.PERMISSION_READ))
        {
            return;
        }

        // Init controls
        rfvCodeName.ErrorMessage = ResHelper.GetString("general.requirescodename");
        rfvDisplayName.ErrorMessage = ResHelper.GetString("general.requiresdisplayname");
        btnOK.Text = ResHelper.GetString("General.OK");
        if (CMSContext.CurrentSite != null)
        {
            chkAddIndexToCurrentSite.Text = ResHelper.GetString("srch.newindex.addindextocurrentsite") + " " + CMSContext.CurrentSite.DisplayName;
        }
        else
        {
            chkAddIndexToCurrentSite.Visible = false;
        }
        
        if (!RequestHelper.IsPostBack())
        {
            this.ResetControls();
        }
    }


    /// <summary>
    /// Resets all boxes.
    /// </summary>
    public void ResetControls()
    {
        txtCodeName.Text = null;
        txtDisplayName.Text = null;

        //Fill drop down list
        DataHelper.FillWithEnum<AnalyzerTypeEnum>(drpAnalyzer, "srch.index.", SearchIndexInfoProvider.AnalyzerEnumToString);

        drpAnalyzer.SelectedValue =  SearchIndexInfoProvider.AnalyzerEnumToString(AnalyzerTypeEnum.StandardAnalyzer);
        chkAddIndexToCurrentSite.Checked = true;
    }


    /// <summary>
    /// Sets data to database.
    /// </summary>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        if (!CheckPermissions("cms.searchindex", CMSAdminControl.PERMISSION_MODIFY))
        {
            return;
        }

        // Get code name
        string codeName = ValidationHelper.GetCodeName(txtCodeName.Text, null, null);

        // Perform validation
        string errorMessage = new Validator().NotEmpty(codeName, rfvCodeName.ErrorMessage)
            .NotEmpty(txtDisplayName.Text, rfvDisplayName.ErrorMessage).Result;

        // Check CodeName for identificator format
        if (!ValidationHelper.IsCodeName(codeName))
        {
            errorMessage = ResHelper.GetString("General.ErrorCodeNameInIdentificatorFormat");
        }

        if (errorMessage == "")
        {
            // Create new 
            SearchIndexInfo sii = SearchIndexInfoProvider.GetSearchIndexInfo(codeName);

            if (sii == null)
            {
                sii = new SearchIndexInfo();

                // Set the fields
                sii.IndexName = codeName;
                sii.IndexDisplayName = txtDisplayName.Text.Trim();
                sii.IndexAnalyzerType = SearchIndexInfoProvider.AnalyzerCodenameToEnum(drpAnalyzer.SelectedValue);
                // Community indexing is not yet supported
                //sii.IndexIsCommunityGroup = chkCommunity.Checked;
                sii.IndexIsCommunityGroup = false;
                sii.IndexType = PredefinedObjectType.DOCUMENT;

                // Save the object
                SearchIndexInfoProvider.SetSearchIndexInfo(sii);
                this.ItemID = sii.IndexID;

                // Assing to current website
                if (chkAddIndexToCurrentSite.Checked)
                {
                    SearchIndexSiteInfoProvider.AddSearchIndexToSite(sii.IndexID, CMSContext.CurrentSiteID);
                }

                // Redirect to edit mode
                RaiseOnSaved();
            }
            else
            {
                // Error message - code name already exists
                lblError.Visible = true;
                lblError.Text = ResHelper.GetString("srch.index.codenameexists");
            }
        }
        else
        {
            // Error message - validation
            lblError.Visible = true;
            lblError.Text = errorMessage;
        }
    }
}
