using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Net.Mail;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.SettingsProvider;
using CMS.UIControls;
using CMS.EmailEngine;
using CMS.SiteProvider;
using CMS.FormControls;

public partial class CMSSiteManager_Administration_Users_General_User_MassEmail : AdministrationPage
{
    private int siteId = 0;
    private UserInfo currentUser = null;
    private FormEngineUserControl groupsControl = null;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Check "read" permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.Users", "Read"))
        {
            RedirectToAccessDenied("CMS.Users", "Read");
        }

        this.CurrentMaster.DisplayControlsPanel = true;
        this.drpSites.SelectedIndexChanged += new EventHandler(drpSites_SelectedIndexChanged);

        currentUser = CMSContext.CurrentUser;

        siteId = QueryHelper.GetInteger("siteId", ValidationHelper.GetInteger(this.drpSites.SelectedValue, 0));

        // Disable sites dropdown in cmsdesk
        if (siteId != 0)
        {
            this.plcRoleGroup.Visible = true;
            this.CurrentMaster.DisplayControlsPanel = false;
            try
            {
                this.drpSites.SelectedValue = siteId.ToString();
            }
            catch { }
        }

        if (!RequestHelper.IsPostBack())
        {
            LoadSites();
            if (currentUser != null)
            {
                txtFrom.Text = CMSContext.CurrentUser.Email;
            }
        }

        this.CurrentMaster.Title.TitleImage = UrlHelper.ImagesDirectory + "/CMSSiteManager/Administration/Users/massemail.gif";
        this.CurrentMaster.Title.TitleText = ResHelper.GetString("administration.users_header.massemails");

        this.CurrentMaster.Title.HelpTopicName = "User_MassEmail";

        this.lblSite.Text = ResHelper.GetString("general.site") + ResHelper.Colon;
        this.lblFrom.Text = ResHelper.GetString("general.fromemail") + ResHelper.Colon;
        this.lblGroups.Text = ResHelper.GetString("general.groups") + ResHelper.Colon;
        this.lblRoles.Text = ResHelper.GetString("general.roles") + ResHelper.Colon;
        this.lblSubject.Text = ResHelper.GetString("general.subject") + ResHelper.Colon;
        this.lblUsers.Text = ResHelper.GetString("general.users") + ResHelper.Colon;
        this.btnSend.Text = ResHelper.GetString("general.send");
        this.uploader.AddButtonImagePath = UrlHelper.ImagesDirectory + "/CMSSiteManager/Administration/Users/addattachment.gif";

        this.pnlRecipients.GroupingText = ResHelper.GetString("userlist.recipients");
        this.pnlAttachments.GroupingText = ResHelper.GetString("general.attachments");

        this.btnSend.Click += new EventHandler(btnSend_Click);

        bool showGroups = false;
        this.plcGroups.Visible = false;

        if (siteId > 0)
        {
            SiteInfo si = SiteInfoProvider.GetSiteInfo(siteId);
            if (si != null)
            {
                showGroups = ModuleCommands.CommunitySiteHasGroup(si.SiteID);
            }
        }

        if (showGroups)
        {
            this.plcGroups.Visible = true;
            groupsControl = Page.LoadControl("~/CMSModules/Groups/FormControls/MembershipGroupSelector.ascx") as FormEngineUserControl;
            if (groupsControl != null)
            {
                this.plcGroupSelector.Controls.Add(groupsControl);
                this.groupsControl.FormControlParameter = siteId;
                this.groupsControl.IsLiveSite = false;
                this.groupsControl.SetValue("UseFriendlyMode", true); 
            }
        }

        this.users.SiteID = siteId;
        this.roles.SiteID = siteId;

        this.Page.ClientScript.RegisterClientScriptBlock(typeof(string), "IsSubjectEmpty", ScriptHelper.GetScript(@"
        function IsEmailEmpty()
        {
            var obj = document.getElementById('" + txtSubject.ClientID + @"');
            if (obj != null)
            {
                if ((obj.value == null) || obj.value == '' || obj.value.replace(/ /g,'') == '')
                {
                    if (!confirm(" + ScriptHelper.GetString(ResHelper.GetString("massemail.emptyemailsubject")) + @"))
                    {
                        return false;
                    }
                }
            }

            return true;
        }
        "));

        btnSend.OnClientClick = "return IsEmailEmpty()";

        // Hides HTML or text area according to e-mail format in settings
        EnsureEmailFormatRegions();
        
        // Initialize the e-mail text editor
        InitHTMLEditor();
    }


    /// <summary>
    /// Site Dropdown change event.
    /// </summary>
    protected void drpSites_SelectedIndexChanged(object sender, EventArgs e)
    {
        siteId = ValidationHelper.GetInteger(drpSites.SelectedValue, 0);
        this.plcRoleGroup.Visible = (siteId != 0);
        this.roles.SiteID = siteId;
        this.users.SiteID = siteId;

        if (siteId > 0)
        {
            SiteInfo si = SiteInfoProvider.GetSiteInfo(siteId);
            if ((si != null) && (ModuleCommands.CommunitySiteHasGroup(si.SiteID)))
            {
                if (groupsControl != null)
                {
                    this.plcGroups.Visible = true;
                    this.groupsControl.FormControlParameter = siteId;
                }
            }
            else
            {
                this.plcGroups.Visible = false;
            }
        }

        // Hides HTML or text area according to e-mail format in settings
        EnsureEmailFormatRegions();
    }


    /// <summary>
    /// Loads the site dropdowlist.
    /// </summary>
    private void LoadSites()
    {
        if (currentUser != null)
        {
            if (currentUser.IsGlobalAdministrator)
            {
                this.drpSites.DataSource = SiteInfoProvider.GetAllSites();
            }
            else
            {
                this.drpSites.DataSource = UserInfoProvider.GetUserSites(currentUser.UserID);
            }
            this.drpSites.DataBind();
            this.drpSites.Items.Insert(0, new ListItem(ResHelper.GetString("general.selectall"), "0"));
        }
    }


    /// <summary>
    /// Sends the email.
    /// </summary>
    protected void btnSend_Click(object sender, EventArgs e)
    {
        // Check "modify" permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.Users", "Modify"))
        {
            RedirectToAccessDenied("CMS.Users", "Modify");
        }

        // Validate first
        string errorMessage = new Validator().IsEmail(this.txtFrom.Text, ResHelper.GetString("general.correctemailformat")).Result;

        // Get recipients
        string groupNames = "";
        if (groupsControl != null)
        {
            groupNames = Convert.ToString(this.groupsControl.Value);
        } 
        string userNames = Convert.ToString(this.users.Value);
        string roleNames = Convert.ToString(this.roles.Value);

        if (string.IsNullOrEmpty(groupNames) && string.IsNullOrEmpty(userNames) && string.IsNullOrEmpty(roleNames))
        {
            errorMessage = ResHelper.GetString("massemail.norecipients");
        }

        if (errorMessage != "")
        {
            this.lblError.Text = errorMessage;
            this.lblError.Visible = true;
            return;
        }

        // Create the message
        EmailMessage message = new EmailMessage();
        message.Subject = this.txtSubject.Text;
        message.From = this.txtFrom.Text;
        if (plcText.Visible)
        {
            message.Body = htmlText.ResolvedValue;
        }
        if (plcPlainText.Visible)
        {
            message.PlainTextBody = txtPlainText.Text;
        }

        // Get the attachments
        HttpPostedFile[] attachments = this.uploader.PostedFiles;
        foreach (HttpPostedFile att in attachments)
        {
            message.Attachments.Add(new Attachment(att.InputStream, att.FileName));
        }

        // Send the message using email engine
        EmailSender.SendMassEmail(message, userNames, roleNames, groupNames, siteId);

        this.lblInfo.Text = ResHelper.GetString("massemail.emailsent");
        this.lblInfo.Visible = true;
    }


    /// <summary>
    /// Initializes HTML editor's settings
    /// </summary>
    protected void InitHTMLEditor()
    {
        this.htmlText.BasePath = "~/CMSAdminControls/FCKeditor/";
        this.htmlText.AutoDetectLanguage = false;
        this.htmlText.DefaultLanguage = System.Threading.Thread.CurrentThread.CurrentCulture.TwoLetterISOLanguageName;
        this.htmlText.ToolbarSet = "SimpleEdit";
        this.htmlText.MediaDialogConfig.UseFullURL = true;
        this.htmlText.LinkDialogConfig.UseFullURL = true;
        this.htmlText.QuickInsertConfig.UseFullURL = true;
    }


    /// <summary>
    /// Hides HTML or text area according to e-mail format in settings
    /// </summary>
    protected void EnsureEmailFormatRegions()
    {
        string siteName = null;
        plcPlainText.Visible = true;
        plcText.Visible = true;

        if (siteId > 0)
        {
            // Get site info object to reach site name
            SiteInfo site = SiteInfoProvider.GetSiteInfo(siteId);
            if (site != null)
            {
                siteName = site.SiteName;
            }
        }

        // Get e-mail format from settings
        EmailFormatEnum emailFormat = EmailHelper.GetEmailFormat(SettingsKeyProvider.GetStringValue(siteName + ".CMSEmailFormat"), EmailFormatEnum.Html);
        switch (emailFormat)
        {
            case EmailFormatEnum.Html:
                plcPlainText.Visible = false;
                break;

            case EmailFormatEnum.PlainText:
                plcText.Visible = false;
                break;
        }
    }
}
